library nui_map;
import 'dart:typed_data';

import 'package:flutter/cupertino.dart';
import 'package:google_maps_cluster_manager/google_maps_cluster_manager.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_geolocator/nui_geolocator.dart';

abstract class NUIMapComponent{
  String id;
  LatLng getMapPosition();
}

typedef NUIMapMarkerOnCluster = NUIMapMarker Function(List<NUIMapMarker> clusterItems);

class NUIMapClusterItem extends ClusterItem<NUIMapMarker>{
  NUIMapClusterItem(LatLng location, NUIMapMarker marker) : super(location, item: marker);
}

class NUIMapMarker<T> extends NUIMapComponent{
  NUIMapMarker({String id, this.data, @required this.position, this.label, this.draggable: false, this.anchor: const Offset(0.5, 1.0), this.customMarker, this.customFutureMarker, this.markerStyle : BitmapDescriptor.defaultMarker, this.onClusterListener}){
    this.id = id;
  }

  T data;
  NUILatLng position;
  String label;
  bool draggable;
  Uint8List customMarker;
  Offset anchor;
  Future<Uint8List> customFutureMarker;
  BitmapDescriptor markerStyle;
  NUIMapMarkerOnCluster onClusterListener;

  @override
  LatLng getMapPosition() {
    return LatLng(position.latitude, position.longitude);
  }

  Future<Marker> getClusteredMarker(List<NUIMapMarker> markers, {Function onTap}) async{
    if(onClusterListener != null) {
      final marker = onClusterListener(markers);
      return marker.getMarker(onTap: onTap);
    }
    return null;
  }

  Future<Marker> getMarker({Function onTap}) async{
    Uint8List imageToUse = customMarker;
    if(customFutureMarker != null){
      logNUI("NUIMap", "Custom future marker is found, loading that ...");
      imageToUse = await customFutureMarker;
      logNUI("NUIMap", "Custom future marker loaded successfully");
    }

    return Marker(
        markerId: MarkerId(id ?? randomUUID()),
        position: getMapPosition(),
        anchor: anchor,
        draggable: draggable ?? false,
        onTap: onTap,
        icon: imageToUse != null ? BitmapDescriptor.fromBytes(imageToUse) : (markerStyle ?? BitmapDescriptor.defaultMarker),
        infoWindow: isNullOrEmpty(label) ? null : InfoWindow(title: label),
    );
  }
}

class NUIMapCircle extends NUIMapComponent{
  NUIMapCircle({String id, @required this.position, @required this.radiusInMetre, this.strokeWidth : 0, this.strokeColor, this.fillColor}){
    this.id = id;
  }
  final NUILatLng position;
  final double radiusInMetre;
  final Color fillColor;
  final Color strokeColor;
  final int strokeWidth;
  
  @override
  LatLng getMapPosition() {
    return LatLng(position.latitude, position.longitude);
  }
  
  Circle getCircle(){
    return Circle(
        circleId: CircleId(id ?? randomUUID()),
        center: LatLng(position.latitude, position.longitude),
        radius: radiusInMetre ?? 100,
        strokeColor: strokeColor ?? NUIColors.NUIAccent.withOpacity(0.3),
        fillColor: fillColor ?? NUIColors.NUIAccent.withOpacity(0.3),
        strokeWidth: strokeWidth ?? 0
    );
  }

}

class NUIMapPolygon{

}

class NUIMap {
  static void initialize(){
    NUIGeoLocatorBuilder()
        .build();
  }
}

